import glob
import os

import importlib_resources

from pydirac.analysis.utility import *
from pydirac.io.outputs import Output

data_root = importlib_resources.files("pydirac.tests.data")
He_q_so_dir = str(data_root / "He_q_so")
He_q_mrci_dir = str(data_root / "He_q_mrci")
He_so_dir = str(data_root / "He_so")
He_nr_dir = str(data_root / "He_nr")
He_mrci_dir = str(data_root / "He_mrci")
He_q_theta_dir = str(data_root / "He_q_theta")


def test_get_keyword():
    assert get_keyword("C", "energy", "(core 2)[vir 3]") == "C@energy@(core 2)[vir 3]"
    assert get_keyword("H", "gradient", "(occ 3)[vir 2]") == "H@gradient@(occ 3)[vir 2]"
    assert get_keyword("O", "hessian", "(occ 4)[vir 4]") == "O@hessian@(occ 4)[vir 4]"


def test_get_orbital_info():
    assert get_orbital_info(2, 3) == "(core 2)[vir 3]"
    assert get_orbital_info(3, 2) == "(core 3)[vir 2]"
    assert get_orbital_info(4, 4) == "(core 4)[vir 4]"


def test_get_energy():
    current = os.getcwd()
    os.chdir(He_q_so_dir)
    for calc_dir in glob.glob("*"):
        if os.path.isdir(calc_dir):
            os.chdir(os.path.join(He_q_so_dir, calc_dir))
            if "JOB_DONE" in glob.glob("*"):
                outfiles = glob.glob("*.out")
                if len(outfiles) > 1:
                    raise RuntimeError(
                        "There are two output file in current " "directory: {}".format(calc_dir)
                    )
                else:
                    outfile = outfiles[0]
                    energy = get_energy(outfile, method="CCSD(T)")
                    print(f"{calc_dir} CCSD(T): {energy}")
            os.chdir("..")
    os.chdir(current)


def test_output_object():
    current = os.getcwd()
    os.chdir(He_q_so_dir)
    for calc_dir in glob.glob("*"):
        if os.path.isdir(calc_dir):
            os.chdir(os.path.join(He_q_so_dir, calc_dir))
            if "JOB_DONE" in glob.glob("*"):
                outfiles = glob.glob("*.out")
                if len(outfiles) > 1:
                    raise RuntimeError(
                        "There are two output file in current " "directory: {}".format(calc_dir)
                    )
                else:
                    outfile = outfiles[0]
                    output_obj = Output(outfile)
                    settings = output_obj.parse_input()
            os.chdir("..")
    os.chdir(current)


def test_output_object_CI():
    current = os.getcwd()
    os.chdir(He_mrci_dir)
    for calc_dir in glob.glob("dyall*"):
        if os.path.isdir(calc_dir):
            os.chdir(os.path.join(He_mrci_dir, calc_dir))
            if "JOB_DONE" in glob.glob("*"):
                outfiles = glob.glob("*.out")
                if len(outfiles) > 1:
                    raise RuntimeError(
                        "There are two output file in current " "directory: {}".format(calc_dir)
                    )
                else:
                    outfile = outfiles[0]
                    output_obj = Output(outfile)
                    res = output_obj.parse_results()
                    print(output_obj.energy_settings)

                    res_dict = output_obj.as_dict()
                    print(res_dict)

            os.chdir("..")
    os.chdir(current)



import importlib_resources
import pytest

from pydirac.io.outputs import Output

data_root = importlib_resources.files("pydirac.tests.data")
K_mrci_dir = data_root / "K_mrci"

# out_fn1 = str(data_root / "Li" / "Li_D-CC-SR.out")
# out_fn2 = str(data_root / "Li" / "Li_D-CC-SO.out")
# ref_dict = {out_fn1: [162, 2, 2, 158], out_fn2: [212, 2, 2, 208]}
#
# @pytest.mark.parametrize("out_fn", [out_fn1, out_fn2])
# @pytest.mark.parametrize("e_min", [-10.0])
# @pytest.mark.parametrize("e_max", [10.0])
# def test_get_orbital_info(out_fn, e_min, e_max):
#     out = Output(filename=out_fn)
#     out.parse_orbit()
#     nb_tot = out.mos.nao(e_min, e_max)
#     nb_occ = out.mos.nb_closed_ao(e_min, e_max)
#     nb_open = out.mos.nb_open_ao(e_min, e_max)
#     nb_vir = out.mos.nb_virtual_ao(e_min, e_max)
#     assert nb_tot == nb_occ + nb_open + nb_vir
#     assert ref_dict[out_fn] == [nb_tot, nb_occ, nb_open, nb_vir]


def test_relcc():
    out_fn = str(
        data_root
        / "He_q_so"
        / "d-aug-dyall.acv3z_+0.00001"
        / "He_d-aug-dyall.acv3z_He_d-aug-dyall.acv3z_zff=+0.00001.out"
    )
    o = Output(out_fn)
    res = o.as_dict()
    assert res["energy_settings"]["scf_e"] == pytest.approx(-2.861794767985585)
    assert res["energy_settings"]["mp2_e"] == pytest.approx(-2.895006044817857)
    assert res["energy_settings"]["ccsd_e"] == pytest.approx(-2.900862077849925)
    assert res["energy_settings"]["ccsd(t)_e"] == pytest.approx(-2.900862077849925)
    assert res["task_type"] == "Q-4C-DC-CC@d-aug-dyall.acv3z"


def test_mrci():
    out_fn = str(
        data_root / "He_mrci" / "dyall.acv4z_+0.001/He_dyall.acv4z_He_dyall.acv4z_zff=+0.001.out"
    )
    o = Output(out_fn)
    res = o.as_dict()
    assert res["energy_settings"]["scf_e"] == pytest.approx(-2.8618113380597565)
    assert res["energy_settings"]["ci_e"]["sym_1_root_1"] == pytest.approx(-2.8975548136776)
    assert res["task_type"] == "D-4C-DC-CI@dyall.acv4z"



import importlib_resources
import pytest

from pydirac import Settings
from pydirac.analysis.polarizability import get_polarizability

data_root = importlib_resources.files("pydirac.tests.data")
He_q_so_dir = str(data_root / "He_q_so")
He_q_mrci_dir = str(data_root / "He_q_mrci")
He_so_dir = str(data_root / "He_so")
He_nr_dir = str(data_root / "He_nr")
He_mrci_dir = str(data_root / "He_mrci")
He_q_theta_dir = str(data_root / "He_q_theta")


@pytest.mark.parametrize(
    "dirout_and_deepth",
    [
        (He_nr_dir, 1),
        (He_so_dir, 0),
        (He_mrci_dir, 0),
        (He_q_theta_dir, 1),
        (He_q_so_dir, 0),
        (He_q_mrci_dir, 0),
    ],
)
def test_calc_dipole_polarizability(dirout_and_deepth):
    res = get_polarizability(dirout_and_deepth[0], deepth=dirout_and_deepth[1])
    out = Settings(res)
    print(out)



import importlib_resources

from pydirac.core.settings import Settings
from pydirac.io.inputs import Inp


def test_read_func():
    dirac_inp = str(
        importlib_resources.files("pydirac.tests.data")
        / "He_mrci"
        / "dyall.acv4z_+0.001/He_dyall.acv4z.inp"
    )
    input = Inp.from_file(dirac_inp)
    print(input["DIRAC"])
    assert input["DIRAC"]["TITLE"] == [" He, DOSSSS, SCF"]
    assert input["DIRAC"]["ANALYZE"] == True
    assert input["DIRAC"]["WAVE F"] == True

    assert input["ANALYZE"]["MULPOP"]["_en"] == True
    assert input["ANALYZE"]["MULPOP"]["VECPOP"] == ["1..oo"]

    assert input["HAMILTONIAN"]["DOSSSS"] == True
    # not strip, so the space before keywords are kept
    assert input["HAMILTONIAN"]["OPERATOR"] == [" ZDIPLEN", " COMFACTOR", " zff"]
    assert input["INTEGRALS"]["READINP"]["UNCONTRACT"] == True

    assert input["WAVE FUNCTIONS"]["KR CI"] == True
    assert input["WAVE FUNCTIONS"]["RESOLVE"] == True
    assert input["WAVE FUNCTIONS"]["SCF"]["_en"] == True

    assert input["WAVE FUNCTIONS"]["SCF"]["CLOSED SHELL"] == ["2"]
    assert input["WAVE FUNCTIONS"]["SCF"]["EVCCNV"] == ["1.0D-9  5.0D-8"]
    assert input["WAVE FUNCTIONS"]["SCF"]["MAXITR"] == ["60"]

    assert input["WAVE FUNCTIONS"]["KRCICALC"]["CI PROGRAM"] == ["LUCIAREL"]
    assert input["WAVE FUNCTIONS"]["KRCICALC"]["INACTIVE"] == ["0"]
    assert input["WAVE FUNCTIONS"]["KRCICALC"]["GAS SHELLS"] == [
        "2",
        "0 2 / 1",
        "2 2 / 30",
    ]

    assert input["WAVE FUNCTIONS"]["KRCICALC"]["MAX CI"] == ["120"]
    assert input["WAVE FUNCTIONS"]["KRCICALC"]["MXCIVE"] == ["60"]
    assert input["WAVE FUNCTIONS"]["KRCICALC"]["NOOCCN"] == True
    assert input["WAVE FUNCTIONS"]["KRCICALC"]["RSTRCI"] == ["rstr"]


def test_write():
    # set single calculation parameters (single point, TZ2P/PW91)
    sett = Settings()
    sett.dirac.title = "B, DOSSSS, KRCI"
    sett.dirac.analyze = True
    wf_tag = "WAVE FUNCTIONS"
    sett.set_nested(("DIRAC", wf_tag), True)
    sett.analyze.mulpop._en = True
    sett.analyze.mulpop.vecpop = "1..oo"
    sett.hamiltonian.dossss = False
    sett.hamiltonian.x2c = True
    sett.hamiltonian.nospin = True
    sett.hamiltonian.operator = [" ZDIPLEN", " COMFACTOR", " 0.01"]
    sett.integrals.readinp.uncontract = True
    sett.general.pcmout = True

    wave_func = Settings()
    wave_func["KR CI"] = True
    wave_func.resolve = True

    scf = Settings()
    scf._en = True
    scf["CLOSED SHELL"] = 4
    scf["OPEN SHELL"] = [1, "1/6"]
    scf.evccnv = "1.0D-9 5.0D-8"
    scf.maxitr = 90

    wave_func.scf = scf
    sett[wf_tag] = wave_func

    krci = Settings()
    krci["CI PROGRAM"] = "LUCIAREL"
    krci["INACTIVE"] = 1
    krci["GAS SHELLS"] = [3, "0 2 / 1", "1 3 / 3", "3 3 / 10"]
    krci["MAX CI"] = 60
    krci["NOOCCN"] = True
    krci["DIPMOM"] = True
    krci["RSTRCI"] = 0
    krci["CIROOTS_id_0"] = "3  3"
    krci["CIROOTS_id_1"] = "4  3"
    sett.set_nested(("WAVE FUNCTIONS", "KRCICALC"), krci)

    job = Inp(sett.as_dict())

    ref_str = """**DIRAC
.TITLE
B, DOSSSS, KRCI
.ANALYZE
.WAVE FUNCTIONS
**ANALYZE
.MULPOP
*MULPOP
.VECPOP
1..oo
**HAMILTONIAN
.X2C
.NOSPIN
.OPERATOR
 ZDIPLEN
 COMFACTOR
 0.01
**INTEGRALS
*READINP
.UNCONTRACT
**GENERAL
.PCMOUT
**WAVE FUNCTIONS
.KR CI
.RESOLVE
.SCF
*SCF
.CLOSED SHELL
4
.OPEN SHELL
1
1/6
.EVCCNV
1.0D-9 5.0D-8
.MAXITR
90
*KRCICALC
.CI PROGRAM
LUCIAREL
.INACTIVE
1
.GAS SHELLS
3
0 2 / 1
1 3 / 3
3 3 / 10
.MAX CI
60
.NOOCCN
.DIPMOM
.RSTRCI
0
.CIROOTS
3  3
.CIROOTS
4  3
*END OF INPUT
"""
    assert str(job) == ref_str



import importlib_resources
import pytest

from pydirac.core.molecule import Molecule
from pydirac.io.sets import *

dir_out_He = str(
    importlib_resources.files("pydirac.tests.data.He_q_so") / "d-aug-dyall.acv3z_+0.00001"
)


@pytest.fixture
def cc_set():
    return AtomicCCSet(molecule=Molecule(["Mc"], [[0.0, 0.0, 0.0]]))


# @pytest.mark.skip(reason="not implemented yet")
def test_atomic_cc_set_inp(cc_set):
    inp_from_code = cc_set.inp
    inp_from_dict = {
        "DIRAC": {
            "TITLE": "DIRAC input file generated by PYDIRAC",
            "WAVE F": True,
            "ANALYZE": True,
            "4INDEX": True,
        },
        "ANALYZE": {
            "MULPOP": {
                "_en": True,
                "VECPOP": "1..oo",
            }
        },
        "HAMILTONIAN": {
            "DOSSSS": True,
            "X2C": False,
            "NOSPIN": False,
        },
        "INTEGRALS": {
            "READINP": "UNCONTRACT",
        },
        "WAVE FUNCTIONS": {
            "RESOLVE": True,
            "SCF": {
                "_en": True,
                "EVCCNV": "1.0D-9 5.0D-8",
                "MAXITR": 60,
                "CLOSED SHELL": "112",
                "OPEN SHELL": "1\n3/6",
            },
            "RELCCSD": True,
        },
        "MOLTRA": {
            "ACTIVE": "-20.0 25.0 0.01",
        },
        "RELCC": {
            "ENERGY": True,
            "PRINT": 1,
            "CCENER": True,
            "MAXIT": 60,
            "NTOL": 10,
        },
    }
    assert inp_from_code == inp_from_dict


def test_atomic_cc_set_mol(cc_set):
    mol_from_code = {
        "basis_type": cc_set.mol.basis_type,
        "basis_lib": cc_set.mol.basis_lib,
    }
    mol_from_dict = {
        "basis_type": "dyall.v3z",
        "basis_lib": "BASIS",
    }
    assert mol_from_code == mol_from_dict


def test_atomic_cc_set_from_prev_calc():
    new_cc = AtomicCCSet.from_prev_calc(
        dir_out_He, no_scf=True, user_mol_settings={"basis_type": "s-aug-ANO-RCC"}
    )
    assert new_cc.inp is not None
    assert new_cc.mol is not None



import os
import tempfile

from pydirac.core.basis import get_custom_basis_from_ele


def test_basis():
    filename = "B_c-dyall.acv4z.mol"
    with tempfile.TemporaryDirectory() as tmpdirname:
        filepath = os.path.join(tmpdirname, filename)
        get_custom_basis_from_ele("B", "acv4z", filepath)



# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#  Pydirac: PYthon tool for DIRAC software.
#  Copyright (C) 2020-2020 The Pydirac Development Team
#
#  This file is part of Pydirac.
#
#  Pydirac is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 3
#  of the License, or (at your option) any later version.
#
#  Pydirac is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>
#
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
